<?php

class WECR_Show_Content{

	function __construct() {
		add_action( 'woocommerce_before_my_account', array( $this, 'show_content' ) );
	}

	/**
	 * Shows the restricted content that the user has access to
	 *
	 * @since 1.2.2
	 * @author SFNdesign, Curtis McHale
	 * @access public
	 *
	 * @uses $this->get_orders()                    Gets the orders for the current user
	 * @uses $this->get_products_for_order()        Returns the proudcts assigned to an order
	 * @uses $this->get_restricted_content()        Gets the pages/posts that go with the restricted content
	 * @uses get_the_permalink()                    Returns permalink given object_id
	 * @uses get_the_title()                        Returns the title give the object_id
	 */
	public function show_content(){

		$orders = $this->get_orders();

		$products = $this->get_products_for_order( $orders );

		if ( empty( $products ) ){
			return;
		}

		$content = $this->get_restricted_content( $products );
		if ( $content == null ){ return; }

		echo '<h2>';
		_e( 'Your Premium Content', 'wecr' );
		echo '</h2>';

		echo '<ul>';
		foreach( $content as $c ){
			echo '<li><a href="'. get_the_permalink( $c->ID) .'">'.get_the_title( $c->ID ).'</a></li>';
		}
		echo '</ul>';
	}

	/**
	 * Gets the restricted content that the user has access to
	 *
	 * @since 1.2.2
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param array         $product_ids        required        Array of object_ids for the objects we want to get
	 * @uses get_posts()                                        Gets the posts given the args
	 * @return array|objects    $y                              Array of post objects
	 */
	private function get_restricted_content( $product_ids ){

		$posts = $this->get_all_restricted_content();
		$matches = $this->return_only_matched_posts( $posts, $product_ids );
		$matches = $this->eliminate_based_on_my_account( $matches );

		if ( ! empty( $matches ) && is_array( $matches ) ) {
			$match_args = array(
				'post_type'      => 'any',
				'posts_per_page' => - 1,
				'post__in'       => $matches,
			);

			$y = get_posts( $match_args );
		} else {
			$y = null;
		}

		return $y;

	}

	/**
	 * Removes posts/pages/anything that has the 'remove from my account' checkbox checked
	 *
	 * @since 1.2.4
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param array     $matches    required    Array of matched posts
	 * @uses get_post_meta()                    Returns post_meta given key and post_id
	 * @return array    $posts                  Array of posts we want to include in the my account page
	 */
	private function eliminate_based_on_my_account( $matches ){

		$posts = array();

		foreach( $matches as $m ){

			$include = get_post_meta( absint( $m ), '_wecr_remove_from_my_account', true );

			if ( isset( $include ) && ! empty( $include ) ){
				continue;
			} else {
				$posts[] = absint( $m );
			}

		}

		return $posts;
	}

	/**
	 * Returns only post_ids that have matches with product_ids
	 * @param $posts
	 * @param $product_ids
	 * @return array $matches
	 */
	private function return_only_matched_posts( $posts, $product_ids ){

		$matches = array();

		foreach( $posts as $p ){
			$meta = get_post_meta( absint( $p->ID ), '_wecr_easy_restrict_content' );

			if ( is_array( $meta[0] ) && array_intersect( $meta[0], $product_ids ) ){
				$matches[] = absint( $p->ID );
			}
		}

		return $matches;

	}

	/**
	 * Gets all posts/pages/anything that has been restricted
	 *
	 *
	 * @return array
	 */
	private function get_all_restricted_content(){

		$args = array(
			'post_type' => 'any',
			'posts_per_page' => -1,
			'meta_query' => array(
				array(
					'key' => '_wecr_easy_restrict_content',
					'compare' => 'EXISTS',
				)
			)
		);

		$posts = get_posts( $args );

		return $posts;
	}

	/**
	 * Gets the product ids for an order
	 *
	 * @since 1.2.2
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param array     $orders         required        Array of orders
	 * @uses wp_list_pluck()                            Traverses array and retrieves fields
	 * @uses WC_Order()                                 Builds an order object given order id
	 * @uses WC_Order->get_items()                      Returns items on order
	 * @uses $this->deal_with_subscriptions()           Checks subscription products and if valid adds them to the product_id array
	 * @uses $this->deal_with_regular_products()        Deals with regular products (and variations) and adds to array of ids
	 * @return array    $product_ids                    The ids of the products for the order
	 */
	private function get_products_for_order( $orders ){

		$order_ids = wp_list_pluck( $orders, 'ID' );
		$product_ids = array();

		foreach( $order_ids as $id ){
			$order_obj = new WC_Order( $id );
			$items = $order_obj->get_items();

			foreach( $items as $i ){

				$product_ids = $this->deal_with_subscriptions( $i );

				$product_ids = array_merge( $product_ids, $this->deal_with_regular_products( $i ) );

			}
		}

		return $product_ids;

	} // get_products_for_order

	/**
	 * Reads through the regular products and adds them to the list of possible products to show pages/posts/anything for
	 *
	 * @since 1.2.4
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param   object      $order_object       required                The WC order object
	 * @uses WC_Subscriptions_Product->is_subscription()                Returns true if the given product_id is a subscription
	 * @return array        $product_ids                                The product_ids added to the array
	 */
	private function deal_with_regular_products( $order_object ){

		$product_ids = array();

		if ( class_exists( 'WC_Subcriptions_Product' ) ){
			if ( $subscription_class->is_subscription( $order_object['product_id'] ) ){
				return $product_ids;
			}

			if ( isset( $order_object['variation_id'] ) && $subscription_class->is_subscription( $order_object['product_id'] ) ){
				return $product_ids;
			}
		}

		$product_ids[] = $order_object['product_id'];

		if ( isset( $order_object['variation_id'] ) ){
			$product_ids[] = absint( $order_object['variation_id'] );
		}

		return $product_ids;

	} // deal_with_regular_products

	/**
	 * Works through subscriptions and adds valid subscriptions to the array of product_ids for which we show content
	 *
	 * @since 1.2.4
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @param   object      $order_object       required                The WC order object
	 * @uses WC_Subscriptions_Product->is_subscription()                Returns true if the given product_id is a subscription
	 * @return array        $product_ids                                The product_ids added to the array
	 */
	private function deal_with_subscriptions( $order_object ){

		$product_ids = array();

		if ( class_exists( 'WC_Subcriptions_Product' ) ){

			if ( $subscription_class->is_subscription( $order_object['product_id'] ) && WC_Subscriptions_Manager::user_has_subscription( $current_user->ID, $order_object['product_id'], 'active' ) ) {
				$product_ids[] = absint( $order_object['product_id'] );
			}
			if ( isset( $order_object['variation_id'] ) && $subscription_class->is_subscription( $order_object['variation_id'] ) && WC_Subscriptions_Manager::user_has_subscription( $current_user->ID, $order_object['variation_id'], 'active' ) ) {
				$product_ids[] = absint( $order_object['variation_id'] );
			}

		} else {
			return $product_ids;
		}

		return $product_ids;

	} // deal_with_subscriptions

	/**
	 * Returns the orders for a given user_id
	 *
	 * @since 1.2.2
	 * @author SFNdesign, Curtis McHale
	 * @access private
	 *
	 * @uses get_current_user_id()                  Returns the user_id of the current user
	 * @uses get_posts()                            Returns post objects given args
	 * @return array|object $orders                 Array of order objects
	 */
	private function get_orders(){

		$user_id = get_current_user_id();

		$args = array(
			'posts_per_page' => -1,
			'meta_key'       => '_customer_user',
			'meta_value'     => $user_id,
			'post_type'      => 'shop_order',
			'post_status'    => array( 'wc-completed' ),
		);

		$orders = get_posts( $args );

		return $orders;

	} // get_orders

}
$GLOBALS['wecr_show_content'] = new WECR_Show_Content();

